from typing import Dict, Union, Any

from nidhoggr_core.repository import BaseUserRepo
from nidhoggr_core.response import StatusResponse, ErrorResponse
from nidhoggr_core.user import User
from pydantic import ValidationError

from nidhoggr_requests.core import RequestsRepo


class RequestsUserRepo(BaseUserRepo, RequestsRepo):

    @staticmethod
    def _parse_user(result: Dict[str, Any]) -> Union[ErrorResponse, User]:
        try:
            return User.parse_obj(result)
        except ValidationError as e:
            return ErrorResponse(reason="Received malformed User object from API", exception=e)

    def get_user(self, **kwargs: Dict[str, str]) -> Union[ErrorResponse, User]:
        result = self.fetch(endpoint='/user/get', payload=kwargs)

        if result is ErrorResponse:
            return result

        return self._parse_user(result)

    def check_password(self, *, clean: str, uuid: str) -> Union[ErrorResponse, StatusResponse]:
        payload = {'uuid': uuid, 'password': clean}
        result = self.fetch(endpoint='/user/check_password', payload=payload)

        if result is ErrorResponse:
            return result

        return StatusResponse(**result)

    def save_user(self, *, user: User) -> Union[ErrorResponse, User]:
        payload = user.dict()
        result = self.fetch(endpoint='/user/save', payload=payload)

        if result is ErrorResponse:
            return result

        return self._parse_user(result)



from abc import ABCMeta
from typing import Dict, Any, Union

from nidhoggr_core.response import ErrorResponse
from requests import Session, RequestException


class RequestsRepo(metaclass=ABCMeta):
    _session: Session
    _api_url: str
    _timeout: int

    def __init__(self, *, api_url: str, bearer_token: str, timeout: int = 1):
        session = Session()
        session.headers.update({
            'Authorization': f'Bearer {bearer_token}',
            'Content-Type': 'application/json'
        })
        self._session = session
        self._api_url = api_url
        self._timeout = timeout

    def fetch(self, *, endpoint: str, payload: Dict[str, Any]) -> Union[ErrorResponse, Dict[str, Any]]:
        try:
            response = self._session.post(f'{self._api_url}{endpoint}', json=payload, timeout=self._timeout)
        except RequestException as e:
            return ErrorResponse(reason=f"Failed to fetch {endpoint}", exception=e)

        if response.status_code != 200:
            return ErrorResponse(reason=f"Expected 200, got {response.status_code} for {endpoint}")

        result = response.json()
        if not result:
            return ErrorResponse(reason=f"Response from {endpoint} didn't contain valid JSON")
        return result



from pathlib import Path

from setuptools import setup, find_packages

readme = Path('README.md').read_text()
history = Path('HISTORY.md').read_text()

requirements = Path('requirements.txt').read_text().splitlines()
setup_requirements = ['wheel']
test_requirements = Path('requirements_dev.txt').read_text().splitlines()

version_file = Path('VERSION')
if version_file.exists():
    version = version_file.read_text().strip()
else:
    version = '0.0.0'

setup(
    name="nidhoggr_requests",
    version=version,
    license="MIT",
    description="Requests-based repository implementation for nidhoggr",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    author="Roman Shishkin",
    author_email="spark@uwtech.org",
    url="https://git.mc4ep.org/mc4ep/nidhoggr-requests",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: No Input/Output (Daemon)",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "Topic :: Utilities",
    ],
    keywords=["Minecraft", "Yggdrasil", "Authentication", "Nidhoggr"],
    setup_requires=setup_requirements,
    install_requires=requirements,
    tests_require=test_requirements,
    test_suite='tests',
)



from os import path
from pkg_resources import get_distribution, DistributionNotFound

try:
    _dist = get_distribution('nidhoggr_requests')
    dist_loc = path.normcase(_dist.location)
    here = path.normcase(__file__)
    if not here.startswith(path.join(dist_loc, 'nidhoggr_requests')):
        raise DistributionNotFound
except DistributionNotFound:
    __version__ = 'Please install this project with setup.py'
else:
    __version__ = _dist.version



from typing import Dict, Union

from nidhoggr_core.repository import BaseTextureRepo
from nidhoggr_core.response import ErrorResponse
from nidhoggr_core.texture import TextureType, TextureItem

from nidhoggr_requests.core import RequestsRepo


class RequestsTextureRepo(BaseTextureRepo, RequestsRepo):

    def get(self, *, uuid: str) -> Union[ErrorResponse, Dict[TextureType, TextureItem]]:
        payload = {'uuid': uuid}
        result = self.fetch(endpoint='/texture/get', payload=payload)

        if result is ErrorResponse:
            return result

        return {
            TextureType(kind): TextureItem(url=url)
            for kind, url
            in result.items()
        }
